package io.zipcoder;

import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

import java.util.ArrayList;

import static org.junit.Assert.*;

public class ItemParserTest {

    private String rawSingleItem =    "naMe:Milk;price:3.23;type:Food;expiration:1/25/2016##";

    private String rawSingleItemIrregularSeperatorSample = "naMe:MiLK;price:3.23;type:Food^expiration:1/11/2016##";

    private String rawBrokenSingleItem =    "naMe:Milk;price:3.23;type:Food;expiration:1/25/2016##";

    private String rawMultipleItems = "naMe:Milk;price:3.23;type:Food;expiration:1/25/2016##"
                                      +"naME:BreaD;price:1.23;type:Food;expiration:1/02/2016##"
                                      +"NAMe:BrEAD;price:1.23;type:Food;expiration:2/25/2016##";
    private ItemParser itemParser;

    @Before
    public void setUp(){
        itemParser = new ItemParser();
    }

    @Test
    public void parseRawDataIntoStringArrayTest(){
        Integer expectedArraySize = 3;
        ArrayList<String> items = itemParser.parseRawDataIntoStringArray(rawMultipleItems);
        Integer actualArraySize = items.size();
        assertEquals(expectedArraySize, actualArraySize);
    }

    @Test
    public void parseStringIntoItemTest() throws ItemParseException{
        Item expected = new Item("milk", 3.23, "food","1/25/2016");
        Item actual = itemParser.parseStringIntoItem(rawSingleItem);
        assertEquals(expected.toString(), actual.toString());
    }

    @Test(expected = ItemParseException.class)
    public void parseBrokenStringIntoItemTest() throws ItemParseException{
        itemParser.parseStringIntoItem(rawBrokenSingleItem);
    }

    @Test
    public void findKeyValuePairsInRawItemDataTest(){
        Integer expected = 4;
        Integer actual = itemParser.findKeyValuePairsInRawItemData(rawSingleItem).size();
        assertEquals(expected, actual);
    }

    @Test
    public void findKeyValuePairsInRawItemDataTestIrregular(){
        Integer expected = 4;
        Integer actual = itemParser.findKeyValuePairsInRawItemData(rawSingleItemIrregularSeperatorSample).size();
        assertEquals(expected, actual);
    }

    @Test
    public void correctBreadTest1() {
        ItemParser.SpellingCorrector spellingCorrector = itemParser.new SpellingCorrector();
        String text = "NAMe:BrEAD;price:1.23;type:Food;expiration:1/25/2016";
        String expected = "NAMe:Bread;price:1.23;type:Food;expiration:1/25/2016";
        String actual = spellingCorrector.correctBread(text);
        assertEquals(expected,actual);
    }

    @Test
    public void correctBreadTest2() {
        ItemParser.SpellingCorrector spellingCorrector = itemParser.new SpellingCorrector();
        String text = "BREAD";
        String expected = "Bread";
        String actual = spellingCorrector.correctBread(text);
        assertEquals(expected,actual);
    }

    @Test
    public void correctBreadNameTest1() {
        ItemParser.SpellingCorrector spellingCorrector = itemParser.new SpellingCorrector();
        String text = "NAME";
        String expected = "Name";
        String actual = spellingCorrector.correctName(text);
        assertEquals(expected,actual);
    }

    @Test
    public void correctBreadNameTest2() {
        ItemParser.SpellingCorrector spellingCorrector = itemParser.new SpellingCorrector();
        String text = "NAMe:BrEAD;price:1.23;type:Food;expiration:1/25/2016";
        String expected = "Name:BrEAD;price:1.23;type:Food;expiration:1/25/2016";
        String actual = spellingCorrector.correctName(text);
        assertEquals(expected,actual);
    }

    @Test
    public void correctAllSpellingTest() {
        String text = "NAMe:BrEAD;price:1.23;type:Food;expiration:1/25/2016##naMe:appLEs;" +
                "PRICE:2.50;name:COOKIES;PRicE:2.25;nAMe:MIlK";
        String expected = "Name:Bread;Price:1.23;type:Food;expiration:1/25/2016##Name:Apples;" +
                "Price:2.50;Name:Cookies;Price:2.25;Name:Milk";
        String actual = itemParser.correctSpelling(text);
        assertEquals(expected, actual);
    }

    @Test
    public void countOccurencesOfMilkTest() {
        ItemParser.OccurenceCounter occurenceCounter = itemParser.new OccurenceCounter();
        String text = "NAMe:milk;price:1.23;type:Food;expiration:1/25/2016##naMe:appLEs;" +
                "PRICE:2.50;name:COOKIES;PRicE:2.25;nAMe:MIlK";
        int expected = 2;
        int actual = occurenceCounter.countOccurencesOfMilk(text);
        assertEquals(expected, actual);
    }

    @Test
    public void countOccurencesOfBreadTest() {
        ItemParser.OccurenceCounter occurenceCounter = itemParser.new OccurenceCounter();
        String text = "NAMe:Bread;price:1.23;type:Food;expiration:1/25/2016##naMe:appLEs;" +
                "PRICE:2.50;name:COOKIES;PRicE:2.25;nAMe:MIlK;name:BREAD;name:BrEAd";
        int expected = 3;
        int actual = occurenceCounter.countOccurencesOfBread(text);
        assertEquals(expected, actual);
    }

    @Test
    public void countOccurencesOfApplesTest() {
        ItemParser.OccurenceCounter occurenceCounter = itemParser.new OccurenceCounter();
        String text = "NAMe:Bread;price:1.23;type:Food;expiration:1/25/2016##naMe:appLEs;" +
                "PRICE:2.50;name:COOKIES;PRicE:2.25;nAMe:MIlK;name:BREAD;name:BrEAd";
        int expected = 1;
        int actual = occurenceCounter.countOccurencesOfApples(text);
        assertEquals(expected, actual);
    }

    @Test
    public void countOccurencesOfCookiesTest() {
        ItemParser.OccurenceCounter occurenceCounter = itemParser.new OccurenceCounter();
        String text = "NAMe:Bread;price:1.23;type:Food;expiration:1/25/2016##naMe:appLEs;" +
                "PRICE:2.50;name:COOKIES;PRicE:2.25;nAMe:MIlK;name:BREAD;name:cookies;name:c00kies";
        int expected = 3;
        int actual = occurenceCounter.countOccurencesOfCookies(text);
        assertEquals(expected, actual);
    }

    @Test
    public void countOccurencesOfAllItemsTest1() {
        ItemParser.OccurenceCounter occurenceCounter = itemParser.new OccurenceCounter();
        String text = "NAMe:Bread;price:1.23;type:Food;expiration:1/25/2016##naMe:appLEs;" +
                "PRICE:2.50;name:COOKIES;PRicE:2.25;nAMe:MIlK;name:BREAD;name:cookies;name:c00kies";
        int expected = 3;
        int actual = occurenceCounter.countOccurencesOfAllItemsAlphabeticalOrder(text)[2];
        assertEquals(expected, actual);
    }

    @Test
    public void countOccurencesOfAllItemsTest2() {
        ItemParser.OccurenceCounter occurenceCounter = itemParser.new OccurenceCounter();
        String text = "NAMe:Bread;price:1.23;type:Food;expiration:1/25/2016##naMe:appLEs;" +
                "PRICE:2.50;name:COOKIES;PRicE:2.25;nAMe:MIlK;name:BREAD;name:cookies;name:c00kies";
        int expected = 2;
        int actual = occurenceCounter.countOccurencesOfAllItemsAlphabeticalOrder(text)[1];
        assertEquals(expected, actual);
    }


}
